/**
 * ProfileFerments.cpp is part of bmsapp.
 *
 * bmsapp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * bmsapp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "ProfileFerments.h"
#include "EditProfileFerment.h"
#include "../ui/ui_ProfileFerments.h"
#include "config.h"
#include "bmsapp.h"


ProfileFerments::ProfileFerments(QWidget *parent) : QDialog(parent), ui(new Ui::ProfileFerments)
{
    qDebug() << "ProfileFerments start";

    ui->setupUi(this);
    emit refreshTable();

    setWindowTitle( QString("BMSapp - %1 - Profile Fermentation").arg(VERSIONSTRING) );
}


void ProfileFerments::refreshTable()
{
    QString w;
    QWidget* pWidget;
    QLabel *label;
    QHBoxLayout* pLayout;

    qDebug() << "ProfileFerments reload";

    QSqlQuery query("SELECT * FROM profile_fermentation ORDER BY name");
    const QStringList labels({tr("Name"), tr("Start low"), tr("Start high"), tr("Sensor"), tr("Steps"), tr("Duration"), tr("Edit")});

    ui->tableFerments->setColumnCount(7);
    ui->tableFerments->setColumnWidth(0, 450);	/* Name		*/
    ui->tableFerments->setColumnWidth(1,  90);	/* Min temp	*/
    ui->tableFerments->setColumnWidth(2,  90);	/* Max temp	*/
    ui->tableFerments->setColumnWidth(3,  75);	/* Sensor	*/
    ui->tableFerments->setColumnWidth(4,  75);	/* Steps	*/
    ui->tableFerments->setColumnWidth(5, 120);	/* Duration	*/
    ui->tableFerments->setColumnWidth(6,  80);	/* Edit button	*/
    ui->tableFerments->setRowCount(query.size());
    ui->tableFerments->setHorizontalHeaderLabels(labels);
    ui->tableFerments->verticalHeader()->hide();
    ui->tableFerments->setFixedSize(980 + 24, 640);	/* Even if this is too large, it works */

    QTableWidgetItem *rightitem = new QTableWidgetItem();
    rightitem->setTextAlignment(Qt::AlignRight);

    query.first();
    for (int ridx = 0 ; ridx < query.size() ; ridx++ ) {
	ui->tableFerments->setItem(ridx, 0, new QTableWidgetItem(query.value(2).toString()));	/* Name */

	w = QString("%1 °C").arg(query.value(3).toDouble(), 2, 'f', 1, '0' );      /* Min start temp */
        QTableWidgetItem *item = new QTableWidgetItem(w);
        item->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
        ui->tableFerments->setItem(ridx, 1, item);

	w = QString("%1 °C").arg(query.value(4).toDouble(), 2, 'f', 1, '0' );      /* Max start temp */
        item = new QTableWidgetItem(w);
        item->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
        ui->tableFerments->setItem(ridx, 2, item);

	w = (query.value(5).toInt()) ? tr("Fridge") : tr("Beer");	/* Initial sensor */
	item = new QTableWidgetItem(w);
        item->setTextAlignment(Qt::AlignCenter|Qt::AlignVCenter);
        ui->tableFerments->setItem(ridx, 3, item);

	w = QString("%1").arg(query.value(6).toInt());	/* Number of steps */
	item = new QTableWidgetItem(w);
        item->setTextAlignment(Qt::AlignCenter|Qt::AlignVCenter);
        ui->tableFerments->setItem(ridx, 4, item);

	w = Utils::hours_to_string(query.value(7).toInt());	/* Duration */
	item = new QTableWidgetItem(w);
        item->setTextAlignment(Qt::AlignCenter|Qt::AlignVCenter);
        ui->tableFerments->setItem(ridx, 5, item);

	/* Add the Edit button */
	pWidget = new QWidget();
	QPushButton* btn_edit = new QPushButton();
	btn_edit->setObjectName(QString("%1").arg(query.value(0).toString()));	/* Send record with the button */
	btn_edit->setText(tr("Edit"));
	connect(btn_edit, SIGNAL(clicked()), this, SLOT(on_editButton_clicked()));
	pLayout = new QHBoxLayout(pWidget);
	pLayout->addWidget(btn_edit);
	pLayout->setContentsMargins(5, 0, 5, 0);
	pWidget->setLayout(pLayout);
	ui->tableFerments->setCellWidget(ridx, 6, pWidget);
	query.next();
    }
}


ProfileFerments::~ProfileFerments()
{
    qDebug() << "ProfileFerments done";
    delete ui;
}


void ProfileFerments::edit(int recno)
{
    qDebug() << "ProfileFerments edit:" << recno;

    EditProfileFerment dialog(recno, this);
    /* Signal from editor if a refresh is needed */
    connect(&dialog, SIGNAL(entry_changed()), this, SLOT(refreshTable()));
    dialog.setModal(true);
    dialog.exec();
}


void ProfileFerments::on_editButton_clicked()
{
    QPushButton *pb = qobject_cast<QPushButton *>(QObject::sender());
    int recno = pb->objectName().toInt();
    qDebug() << Q_FUNC_INFO << recno;
    edit(recno);
}


void ProfileFerments::on_insertButton_clicked()
{
    qDebug() << Q_FUNC_INFO;
    edit(-1);
}


void ProfileFerments::on_quitButton_clicked()
{
    emit firstWindow();
}

