/**
 * ProfileStyles.cpp is part of bmsapp.
 *
 * bmsapp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * bmsapp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "ProfileStyles.h"
#include "EditProfileStyle.h"
#include "../ui/ui_ProfileStyles.h"
#include "config.h"
#include "bmsapp.h"


ProfileStyles::ProfileStyles(QWidget *parent) : QDialog(parent), ui(new Ui::ProfileStyles)
{
    qDebug() << "ProfileStyles start";

    ui->setupUi(this);
    emit refreshTable();

    setWindowTitle( QString("BMSapp - %1 - Profile Styles").arg(VERSIONSTRING) );
}


void ProfileStyles::refreshTable()
{
    QString w;
    QWidget* pWidget;
    QLabel *label;
    QHBoxLayout* pLayout;

    qDebug() << "ProfileStyles reload";

    QSqlQuery query("SELECT * FROM profile_styles ORDER BY style_guide,style_letter,name");
    const QStringList labels({tr("Guide"), tr("Gr"), tr("Name"), tr("OG"), tr("OG"), tr("FG"), tr("FG"),
	tr("IBU"), tr("IBU"), tr("EBC"), tr("EBC"), tr("Co2"), tr("Co2"), tr("ABV"), tr("ABV"), tr("Edit")});

    ui->tableStyles->setColumnCount(16);
    ui->tableStyles->setColumnWidth(0, 100);	/* Guide	*/
    ui->tableStyles->setColumnWidth(1,  25);	/* Group	*/
    ui->tableStyles->setColumnWidth(2, 300);	/* Name		*/
    ui->tableStyles->setColumnWidth(3,  50);	/* OG		*/
    ui->tableStyles->setColumnWidth(4,  50);
    ui->tableStyles->setColumnWidth(5,  50);	/* FG		*/
    ui->tableStyles->setColumnWidth(6,  50);
    ui->tableStyles->setColumnWidth(7,  40);	/* IBU		*/
    ui->tableStyles->setColumnWidth(8,  40);
    ui->tableStyles->setColumnWidth(9,  40);	/* EBC		*/
    ui->tableStyles->setColumnWidth(10, 40);
    ui->tableStyles->setColumnWidth(11, 40);	/* Co2		*/
    ui->tableStyles->setColumnWidth(12, 40);
    ui->tableStyles->setColumnWidth(13, 40);	/* ABV		*/
    ui->tableStyles->setColumnWidth(14, 40);
    ui->tableStyles->setColumnWidth(15, 80);	/* Edit button	*/
    ui->tableStyles->setRowCount(query.size());
    ui->tableStyles->setHorizontalHeaderLabels(labels);
    ui->tableStyles->verticalHeader()->hide();
    ui->tableStyles->setFixedSize(1025 + 24, 640);	/* Even if this is too large, it works */

    QTableWidgetItem *rightitem = new QTableWidgetItem();
    rightitem->setTextAlignment(Qt::AlignRight);

    query.first();
    for (int ridx = 0 ; ridx < query.size() ; ridx++ ) {
	ui->tableStyles->setItem(ridx, 0, new QTableWidgetItem(query.value(5).toString()));	/* Guide */
	ui->tableStyles->setItem(ridx, 1, new QTableWidgetItem(query.value(4).toString()));	/* Goup */
	ui->tableStyles->setItem(ridx, 2, new QTableWidgetItem(query.value(1).toString()));	/* Name */

	w = QString("%1").arg(query.value(7).toDouble(), 4, 'f', 3, '0' );      /* OG */
        QTableWidgetItem *item = new QTableWidgetItem(w);
        item->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
        ui->tableStyles->setItem(ridx, 3, item);

	w = QString("%1").arg(query.value(8).toDouble(), 4, 'f', 3, '0' );      /* OG */
        item = new QTableWidgetItem(w);
        item->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
        ui->tableStyles->setItem(ridx, 4, item);

	w = QString("%1").arg(query.value(9).toDouble(), 4, 'f', 3, '0' );      /* FG */
        item = new QTableWidgetItem(w);
        item->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
        ui->tableStyles->setItem(ridx, 5, item);

	w = QString("%1").arg(query.value(10).toDouble(), 4, 'f', 3, '0' );      /* FG */
        item = new QTableWidgetItem(w);
        item->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
        ui->tableStyles->setItem(ridx, 6, item);

	w = QString("%1").arg(query.value(11).toDouble(), 1, 'f', 0, '0' );      /* IBU */
        item = new QTableWidgetItem(w);
        item->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
        ui->tableStyles->setItem(ridx, 7, item);

	w = QString("%1").arg(query.value(12).toDouble(), 1, 'f', 0, '0' );      /* IBU */
        item = new QTableWidgetItem(w);
        item->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
        ui->tableStyles->setItem(ridx, 8, item);

	w = QString("%1").arg(query.value(13).toDouble(), 1, 'f', 0, '0' );      /* EBC */
        item = new QTableWidgetItem(w);
        item->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
        ui->tableStyles->setItem(ridx, 9, item);

	w = QString("%1").arg(query.value(14).toDouble(), 1, 'f', 0, '0' );      /* EBC */
        item = new QTableWidgetItem(w);
        item->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
        ui->tableStyles->setItem(ridx, 10, item);

	w = QString("%1").arg(query.value(15).toDouble(), 2, 'f', 1, '0' );      /* Co2 */
        item = new QTableWidgetItem(w);
        item->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
        ui->tableStyles->setItem(ridx, 11, item);

	w = QString("%1").arg(query.value(16).toDouble(), 2, 'f', 1, '0' );      /* Co2 */
        item = new QTableWidgetItem(w);
        item->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
        ui->tableStyles->setItem(ridx, 12, item);

	w = QString("%1").arg(query.value(17).toDouble(), 2, 'f', 1, '0' );      /* ABV */
        item = new QTableWidgetItem(w);
        item->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
        ui->tableStyles->setItem(ridx, 13, item);

	w = QString("%1").arg(query.value(18).toDouble(), 2, 'f', 1, '0' );      /* ABV */
        item = new QTableWidgetItem(w);
        item->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
        ui->tableStyles->setItem(ridx, 14, item);

	/* Add the Edit button */
	pWidget = new QWidget();
	QPushButton* btn_edit = new QPushButton();
	btn_edit->setObjectName(QString("%1").arg(query.value(0).toString()));	/* Send record with the button */
	btn_edit->setText(tr("Edit"));
	connect(btn_edit, SIGNAL(clicked()), this, SLOT(on_editButton_clicked()));
	pLayout = new QHBoxLayout(pWidget);
	pLayout->addWidget(btn_edit);
	pLayout->setContentsMargins(5, 0, 5, 0);
	pWidget->setLayout(pLayout);
	ui->tableStyles->setCellWidget(ridx, 15, pWidget);
	query.next();
    }

    setWindowTitle( QString("BMSapp - %1 - Profile Styles").arg(VERSIONSTRING) );
}


ProfileStyles::~ProfileStyles()
{
    qDebug() << "ProfileStyles done";
    delete ui;
}


void ProfileStyles::edit(int recno)
{
    qDebug() << "ProfileStyles edit:" << recno;

    EditProfileStyle dialog(recno, this);
    /* Signal from editor if a refresh is needed */
    connect(&dialog, SIGNAL(entry_changed()), this, SLOT(refreshTable()));
    dialog.setModal(true);
    dialog.exec();
}


void ProfileStyles::on_editButton_clicked()
{
    QPushButton *pb = qobject_cast<QPushButton *>(QObject::sender());
    int recno = pb->objectName().toInt();
    qDebug() << Q_FUNC_INFO << recno;
    edit(recno);
}


void ProfileStyles::on_insertButton_clicked()
{
    qDebug() << Q_FUNC_INFO;
    edit(-1);
}


void ProfileStyles::on_quitButton_clicked()
{
    emit firstWindow();
}

